package com.ruoyi.borrow.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import cn.hutool.core.util.StrUtil;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.borrow.domain.TbAssetBorrowRecord;
import com.ruoyi.borrow.service.ITbAssetBorrowRecordService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 资产借用Controller
 * 
 * @author ruoyi
 * @date 2024-01-11
 */
@RestController
@RequestMapping("/asset/borrow")
public class TbAssetBorrowRecordController extends BaseController
{
    @Autowired
    private ITbAssetBorrowRecordService tbAssetBorrowRecordService;

    /**
     * 快速归还
     */
    @PostMapping("/fastReturn/{assetNumber}")
    public AjaxResult fastReturn(@PathVariable("assetNumber") String assetNumber) {
        String errMessage = tbAssetBorrowRecordService.fastReturnByAssetNumber(assetNumber);
        if (StrUtil.isNotBlank(errMessage)) {
            return success(errMessage);
        }
        return success("归还成功");
    }

    /**
     * 查询资产借用列表
     */
    @PreAuthorize("@ss.hasPermi('asset:borrow:list')")
    @GetMapping("/list")
    public TableDataInfo list(TbAssetBorrowRecord tbAssetBorrowRecord)
    {
        startPage();
        List<TbAssetBorrowRecord> list = tbAssetBorrowRecordService.selectTbAssetBorrowRecordList(tbAssetBorrowRecord);
        return getDataTable(list);
    }

    /**
     * 导出资产借用列表
     */
    @PreAuthorize("@ss.hasPermi('asset:borrow:export')")
    @Log(title = "资产借用", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TbAssetBorrowRecord tbAssetBorrowRecord)
    {
        List<TbAssetBorrowRecord> list = tbAssetBorrowRecordService.selectTbAssetBorrowRecordList(tbAssetBorrowRecord);
        ExcelUtil<TbAssetBorrowRecord> util = new ExcelUtil<TbAssetBorrowRecord>(TbAssetBorrowRecord.class);
        util.exportExcel(response, list, "资产借用数据");
    }

    /**
     * 获取资产借用详细信息
     */
    @PreAuthorize("@ss.hasPermi('asset:borrow:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tbAssetBorrowRecordService.selectTbAssetBorrowRecordById(id));
    }

    /**
     * 新增资产借用
     */
    @PreAuthorize("@ss.hasPermi('asset:borrow:add')")
    @Log(title = "资产借用", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TbAssetBorrowRecord tbAssetBorrowRecord)
    {
        return toAjax(tbAssetBorrowRecordService.insertTbAssetBorrowRecord(tbAssetBorrowRecord));
    }

    /**
     * 修改资产借用
     */
    @PreAuthorize("@ss.hasPermi('asset:borrow:edit')")
    @Log(title = "资产借用", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TbAssetBorrowRecord tbAssetBorrowRecord)
    {
        return toAjax(tbAssetBorrowRecordService.updateTbAssetBorrowRecord(tbAssetBorrowRecord));
    }

    /**
     * 删除资产借用
     */
    @PreAuthorize("@ss.hasPermi('asset:borrow:remove')")
    @Log(title = "资产借用", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tbAssetBorrowRecordService.deleteTbAssetBorrowRecordByIds(ids));
    }
}
